/** ***********************************************************************
*
* Veracity Technology Consultants CONFIDENTIAL
* __________________
*
*  2019 Veracity Technology Consultants
*  All Rights Reserved.
*
* NOTICE:  All information contained herein is, and remains
* the property of Veracity Technology Consultants and its suppliers,
* if any.  The intellectual and technical concepts contained
* herein are proprietary to Veracity Technology Consultants
* and its suppliers and may be covered by U.S. and Foreign Patents,
* patents in process, and are protected by trade secret or copyright law.
* Dissemination of this information or reproduction of this material
* is strictly forbidden unless prior written permission is obtained
* from Veracity Technology Consultants.
*/
// const Iterator = require('./analytics/engine/Iterator');
const clone = require('clone');

const QueryStage = require('../../analytics/engine/stages/QueryStage');
const Executor = require('../../analytics/engine/Executor');
const { getUniqueValues, getUniqueActivityDocs, downloadTimeUpdate } = require('./utils');

module.exports = async (lrs, query, courseStatus, res, file) => {
    let timeStart = Date.now();

    // Get first unique row dimension values..
    const divisions = await getUniqueValues(lrs, "object.definition.extensions['https://dcsa-course'].division", query);
    
    // Get all unique pivot column values for time estimation and row entries. 
    const organizationsTotal = await getUniqueValues(lrs, "context.extensions['https://dcsa-user'].organization", query);
    
    if (courseStatus === 'completed') query.$and.push({ "verb.id": "http://adlnet.gov/expapi/verbs/completed" });
    
    res.on('close', () => canceled = true);
    let canceled = false;
    // let total = divisions.length * activityTypesTotal.length * disciplinesTotal.length * activitiesTotal.length * organizationsTotal.length;
    let start = Date.now();
    let iterationCount = 0;
    let total = 0;
    let totalAvg = 0;
    let totalPass = 0;
    let results = [];
    const totalOrgs = organizationsTotal.reduce((a, c) => a = { ...a , [c]: 0}, {});

    // create row template.
    const rowTemplate = {
        Division: '',
        'Activity Type': '',
        'Discipline': '',
        'Activity Title': '',
        'Activity ID': '',
        ...totalOrgs,
        'TOTAL': 0,
    };

    // For column grand totals
    const grandTotal = clone(rowTemplate);
    grandTotal['Division'] = 'Total';

    for (const division of divisions) {
        // file data updates
        if (canceled) break;
        iterationCount++;
        if (totalAvg) downloadTimeUpdate(iterationCount, totalAvg, start, file);

        // For dimension subtotals
        const divTotal = clone(rowTemplate);
        divTotal['Division'] = `TOTAL ${division}`;

        // Create fitler for this dimension given the unique value and get unique values of next row dimension.
        const divFilter = clone(query);
        divFilter.$and.push({ "object.definition.extensions['https://dcsa-course'].division": division });
        const activityTypes = await getUniqueValues(lrs, "object.definition.extensions['https://dcsa-course'].delivery_method", divFilter);

        // Need index to determine what rows have blank values for the row dimensions.
        for (const [atIdx, activityType] of activityTypes.entries()) {
            if (canceled) break;
            iterationCount++;

            const actTotal = clone(rowTemplate);
            actTotal['Activity Type'] = `TOTAL ${activityType}`;

            const actFilter = clone(divFilter);
            actFilter.$and.push({ "object.definition.extensions['https://dcsa-course'].delivery_method": activityType });
            const disciplines = await getUniqueValues(lrs, "object.definition.extensions['https://dcsa-course'].subject_area", actFilter);

            for (const [dcIdx, discipline] of disciplines.entries()) {
                if (canceled) break;
                iterationCount++;

                const discTotal = clone(rowTemplate);
                discTotal['Discipline'] = `TOTAL ${discipline}`;

                const discFilter = clone(actFilter);
                discFilter.$and.push({ "object.definition.extensions['https://dcsa-course'].subject_area": discipline });
                const activities = await getUniqueActivityDocs(lrs, discFilter);

                totalPass++;
                total += (divisions.length * activityTypes.length * disciplines.length * activities.length);
                totalAvg = total / totalPass;

                for (const [aIdx, activity] of activities.entries()) {
                    if (canceled) break;
                    iterationCount++;

                    const actIdFilter = clone(discFilter);
                    actIdFilter.$and.push({ "object.id": activity._id });

                    // Create the row.
                    const row = clone(rowTemplate);
                    row['Division'] = (atIdx === 0 && dcIdx === 0 && aIdx === 0) ? division : '';
                    row['Activity Type'] = (dcIdx === 0 && aIdx === 0) ? activityType : '';
                    row['Discipline'] = aIdx === 0 ? discipline : '';
                    row['Activity Title'] = activity.canonical.display;
                    row['Activity ID'] = activity._id;
                    
                    let rowTotal = 0;

                    let baseQuery = {
                        filter: actIdFilter,
                        process: [
                            {
                                $frequentValues: {
                                    path: "context.extensions['https://dcsa-user'].organization",
                                    limit: 10000,
                                    metrics: {
                                        count: {
                                            $range: "context.extensions['https://dcsa-user'].userid"
                                        }
                                    }
                                }
                            }
                        ]
                    }

                    const innerStage = new QueryStage({
                        filter: baseQuery.filter,
                        process: baseQuery.process,
                        lrs: lrs,
                    });
                    let innerResults = await Executor.runStage(innerStage, null, lrs);

                    if (courseStatus === 'notCompleted') {
                        // baseFilter with just comp
                        const compQuery = clone(baseQuery);
                        compQuery.filter.$and.push({ "verb.id": "http://adlnet.gov/expapi/verbs/completed" });
                        const compStage = new QueryStage({
                            filter: compQuery.filter,
                            process: compQuery.process,
                            lrs: lrs,
                        });
                        const compResults = await Executor.runStage(compStage, null, lrs);

                        // Value is just reg - just comp.
                        innerResults = innerResults.map(reg => {
                            const comp = compResults.find(comp => comp._id === reg._id);
                            return { _id: reg._id, count: comp ? reg.count - comp.count : reg.count }
                        });
                    }

                    for (const innerResult of innerResults) {
                        row[innerResult._id] = innerResult.count;
                        rowTotal += innerResult.count;

                        divTotal[innerResult._id] += innerResult.count;
                        divTotal['TOTAL'] += innerResult.count;

                        actTotal[innerResult._id] += innerResult.count;
                        actTotal['TOTAL'] += innerResult.count;

                        discTotal[innerResult._id] += innerResult.count;
                        discTotal['TOTAL'] += innerResult.count;

                        grandTotal[innerResult._id] += innerResult.count;
                        grandTotal['TOTAL'] += innerResult.count;
                    }

                    // Add row totals.
                    row['TOTAL'] = rowTotal;

                    results.push(row);
                }

                results.push(discTotal);
            }

            results.push(actTotal);
        }

        results.push(divTotal);
    }

    if (canceled) return [];
    if (results.length <= 0) return [];

    // Add column totals.
    results[results.length] = grandTotal;

    console.log('time: ', Date.now() - timeStart);
    return results;
};
