/** ***********************************************************************
*
* Veracity Technology Consultants CONFIDENTIAL
* __________________
*
*  2019 Veracity Technology Consultants
*  All Rights Reserved.
*
* NOTICE:  All information contained herein is, and remains
* the property of Veracity Technology Consultants and its suppliers,
* if any.  The intellectual and technical concepts contained
* herein are proprietary to Veracity Technology Consultants
* and its suppliers and may be covered by U.S. and Foreign Patents,
* patents in process, and are protected by trade secret or copyright law.
* Dissemination of this information or reproduction of this material
* is strictly forbidden unless prior written permission is obtained
* from Veracity Technology Consultants.
*/
// const Iterator = require('./analytics/engine/Iterator');
const clone = require('clone');

const QueryStage = require('../../analytics/engine/stages/QueryStage');
const Executor = require('../../analytics/engine/Executor');
const { getUniqueValues, getUniqueActivityDocs, downloadTimeUpdate } = require('./utils');

module.exports = async (lrs, query, courseStatus, res, file) => {
    let timeStart = Date.now();

     // Get first unique row dimension values..
     const divisions = await getUniqueValues(lrs, "object.definition.extensions['https://dcsa-course'].division", query);
    
    
    if (courseStatus === 'completed') query.$and.push({ "verb.id": "http://adlnet.gov/expapi/verbs/completed" });
    
    res.on('close', () => canceled = true);
    let canceled = false;
    let total = 0;
    let totalAvg = 0;
    let totalPass = 0;
    let start = Date.now();
    let iterationCount = 0;
    
    let results = [];
    const totalCompletions = {
        'Completed': 0,
        'Not Completed': 0,
    };

    // create row template.
    const rowTemplate = {
        Division: '',
        'Discipline': '',
        'Activity Type': '',
        'Activity Title': '',
        'Activity ID': '',
        ...totalCompletions,
        'TOTAL': 0,
    };

    // For column grand totals
    const grandTotal = clone(rowTemplate);
    grandTotal['Division'] = 'Total';

    for (const division of divisions) {
        // file data updates
        if (canceled) break;
        iterationCount++;
        if (totalAvg) downloadTimeUpdate(iterationCount, totalAvg, start, file);

        // For dimension subtotals
        const divTotal = clone(rowTemplate);
        divTotal['Division'] = `TOTAL ${division}`;

        // Create filter for this dimension given the unique value and get unique values of next row dimension.
        const divFilter = clone(query);
        divFilter.$and.push({ "object.definition.extensions['https://dcsa-course'].division": division });
        const disciplines = await getUniqueValues(lrs, "object.definition.extensions['https://dcsa-course'].subject_area", divFilter);

        // Need index to determine what rows have blank values for the row dimensions.
        for (const [dcIdx, discipline] of disciplines.entries()) {
            if (canceled) break;
            iterationCount++;

            const discTotal = clone(rowTemplate);
            discTotal['Discipline'] = `TOTAL ${discipline}`;

            const discFilter = clone(divFilter);
            discFilter.$and.push({ "object.definition.extensions['https://dcsa-course'].subject_area": discipline });
            const activityTypes = await getUniqueValues(lrs, "object.definition.extensions['https://dcsa-course'].delivery_method", discFilter);

            for (const [atIdx, activityType] of activityTypes.entries()) {
                if (canceled) break;
                iterationCount++;

                const actTotal = clone(rowTemplate);
                actTotal['Activity Type'] = `TOTAL ${activityType}`;

                const actFilter = clone(discFilter);
                actFilter.$and.push({ "object.definition.extensions['https://dcsa-course'].delivery_method": activityType });
                const activities = await getUniqueActivityDocs(lrs, actFilter);

                totalPass++;
                total += (divisions.length * activityTypes.length * disciplines.length * activities.length);
                totalAvg = total / totalPass;

                for (const [aIdx, activity] of activities.entries()) {
                    if (canceled) break;
                    iterationCount++;

                    const actIdFilter = clone(discFilter);
                    actIdFilter.$and.push({ "object.id": activity._id });

                    // Create the row.
                    const row = clone(rowTemplate);
                    row['Division'] = (dcIdx === 0 && atIdx === 0 && aIdx === 0) ? division : '';
                    row['Discipline'] = (atIdx === 0 && aIdx === 0) ? discipline : '';
                    row['Activity Type'] = aIdx === 0  ? activityType : '';
                    row['Activity Title'] = activity.canonical.display;
                    row['Activity ID'] = activity._id;

                    let baseQuery = {
                        filter: actIdFilter,
                        process: [
                            {
                                $parallel: {
                                    stages: [
                                        {
                                            $query: {
                                                filter: {},
                                                process: [
                                                    {
                                                        $range: {
                                                            path: "context.extensions['https://dcsa-user'].userid"
                                                        }
                                                    }
                                                ]
                                            }
                                        },
                                        {
                                            $query: {
                                                filter: {
                                                    "verb.id": "http://adlnet.gov/expapi/verbs/completed"
                                                },
                                                process: [
                                                    {
                                                        $range: {
                                                            path: "context.extensions['https://dcsa-user'].userid"
                                                        }
                                                    }
                                                ]
                                            }
                                        }
                                    ]
                                }
                            },
                            {
                                $mergeArrays: { }
                            }
                        ]
                    }

                    const innerStage = new QueryStage({
                        filter: baseQuery.filter,
                        process: baseQuery.process,
                        lrs: lrs,
                    });
                    let innerResults = await Executor.runStage(innerStage, null, lrs);

                    const completed = courseStatus === 'notCompleted' ? 0 : innerResults[1].value;
                    const notCompleted = innerResults[0].value - innerResults[1].value;
                    const all = completed + notCompleted;

                    row['Completed'] = completed;
                    row['Not Completed'] = notCompleted;

                    divTotal['Completed'] += completed;
                    divTotal['Not Completed'] += notCompleted;
                    divTotal['TOTAL'] += all;

                    discTotal['Completed'] += completed;
                    discTotal['Not Completed'] += notCompleted;
                    discTotal['TOTAL'] += all;

                    actTotal['Completed'] += completed;
                    actTotal['Not Completed'] += notCompleted;
                    actTotal['TOTAL'] += all;

                    grandTotal['Completed'] += completed;
                    grandTotal['Not Completed'] += notCompleted;
                    grandTotal['TOTAL'] += all;

                    // Add row totals.
                    row['TOTAL'] = all;

                    results.push(row);
                }

                results.push(actTotal);
            }

            results.push(discTotal);
        }

        results.push(divTotal);
    }

    if (canceled) return [];
    if (results.length <= 0) return [];

    // Add column totals.
    results[results.length] = grandTotal;

    console.log('time: ', Date.now() - timeStart);
    return results;
};
