/** ***********************************************************************
*
* Veracity Technology Consultants CONFIDENTIAL
* __________________
*
*  2019 Veracity Technology Consultants
*  All Rights Reserved.
*
* NOTICE:  All information contained herein is, and remains
* the property of Veracity Technology Consultants and its suppliers,
* if any.  The intellectual and technical concepts contained
* herein are proprietary to Veracity Technology Consultants
* and its suppliers and may be covered by U.S. and Foreign Patents,
* patents in process, and are protected by trade secret or copyright law.
* Dissemination of this information or reproduction of this material
* is strictly forbidden unless prior written permission is obtained
* from Veracity Technology Consultants.
*/
const Iterator = require('./analytics/engine/Iterator');
const { downloadTimeUpdate } = require('./utils');

module.exports = async (lrs, query, courseStatus, res, file) => {
    const cleanValue = (value) => value ? value.trim() : "";
    let timeStart = Date.now();

    let iterator = await (new Iterator(lrs)).find(query);
    let total = await iterator.count();
    iterator = await (new Iterator(lrs)).find(query);

    let canceled = false;
    let start = Date.now();
    res.on('close', () => canceled = true);
    let j = 0;
    
    const resultsMap = { };

    await iterator.iterate(async (i) => {
        if (canceled) return false;
        j++;
        if(j % 1000 == 0 ) {
            downloadTimeUpdate(j, total, start, file);
        }

        if (i.statement.context.extensions && cleanValue(i.statement.context.extensions['https://dcsa-user'].userid)) {
            const key = `${i.statement.object.id}.${i.statement.context.extensions['https://dcsa-user'].userid}`;
            const item = resultsMap[key];
            if (item) {
                if (i.statement.verb.id === "http://adlnet.gov/expapi/verbs/completed") {
                    resultsMap[key]['Course Completion'] = 'Completed'
                }
            } else {
                resultsMap[key] = {
                    'STEPP User ID': i.statement.context.extensions['https://dcsa-user'].userid || '',
                    'Organization': i.statement.context.extensions['https://dcsa-user'].organization || '',
                    'Military Status': i.statement.context.extensions['https://dcsa-user'].status || '',
                    'Country': i.statement.context.extensions['https://dcsa-user'].country || '',
                    'Activity Title': i.statement.object.definition.name.en || '',
                    'Activity ID': i.statement.object.id || '',
                    'Activity Type': i.statement.object.definition.extensions['https://dcsa-course'].delivery_method || '',
                    'Division': i.statement.object.definition.extensions['https://dcsa-course'].division || '',
                    'Discipline': i.statement.object.definition.extensions['https://dcsa-course'].subject_area || '',
                    'Course Completion': i.statement.verb.id === "http://adlnet.gov/expapi/verbs/completed" ? "Completed" : "Not Completed",
                }
            }
        }
    });

    if (canceled) return [];

    let results = Object.keys(resultsMap).map(r => resultsMap[r]);

    results = results.filter(r => {
        switch (courseStatus) {
            case 'completed':
                return r['Course Completion'] === 'Completed';
            case 'notCompleted':
                return r['Course Completion'] === 'Not Completed';
            default:
                return r['Course Completion'];
        }
    });

    results.sort((a, b) => a['STEPP User ID'].localeCompare(b['STEPP User ID']));

    console.log('time: ', Date.now() - timeStart);
    return results;
};
