/** ***********************************************************************
*
* Veracity Technology Consultants CONFIDENTIAL
* __________________
*
*  2019 Veracity Technology Consultants
*  All Rights Reserved.
*
* NOTICE:  All information contained herein is, and remains
* the property of Veracity Technology Consultants and its suppliers,
* if any.  The intellectual and technical concepts contained
* herein are proprietary to Veracity Technology Consultants
* and its suppliers and may be covered by U.S. and Foreign Patents,
* patents in process, and are protected by trade secret or copyright law.
* Dissemination of this information or reproduction of this material
* is strictly forbidden unless prior written permission is obtained
* from Veracity Technology Consultants.
*/
// const Iterator = require('./analytics/engine/Iterator');
const clone = require('clone');

const QueryStage = require('../../analytics/engine/stages/QueryStage');
const Executor = require('../../analytics/engine/Executor');
const { getUniqueValues, getUniqueActivityDocs, downloadTimeUpdate } = require('./utils');

module.exports = async (lrs, query, courseStatus, res, file) => {
    let timeStart = Date.now();
    const organizations = await getUniqueValues(lrs, "context.extensions['https://dcsa-user'].organization", query);

    const statusTotal = await getUniqueValues(lrs, "context.extensions['https://dcsa-user'].status", query);

    if (courseStatus === 'completed') query.$and.push({ "verb.id": "http://adlnet.gov/expapi/verbs/completed" });
    
    res.on('close', () => canceled = true);
    let canceled = false;
    let total = 0;
    let totalAvg = 0;
    let totalPass = 0;
    let start = Date.now();
    let iterationCount = 0;
    
    let results = [];
    const totalStats = statusTotal.reduce((a, c) => a = { ...a , [c]: 0}, {});

    // create row template.
    const rowTemplate = {
        Organization: '',
        ...totalStats,
        'TOTAL': 0,
    };

    // For column grand totals
    const grandTotal = clone(rowTemplate);
    grandTotal['Organization'] = 'Total';

    for (const organization of organizations) {
        // file data updates
        if (canceled) break;
        iterationCount++;
        if (totalAvg) downloadTimeUpdate(iterationCount, totalAvg, start, file);

        const orgFilter = clone(query);
        orgFilter.$and.push({ "context.extensions['https://dcsa-user'].organization": organization });
        const activities = await getUniqueActivityDocs(lrs, orgFilter);

        totalPass++;
        total += (organizations.length * activities.length);
        totalAvg = total / totalPass;

        // Create the row.
        const row = clone(rowTemplate);
        row['Organization'] = organization;

        let rowTotal = 0;

        for (const [aIdx, activity] of activities.entries()) {
            if (canceled) break;
            iterationCount++;

            const actIdFilter = clone(orgFilter);
            actIdFilter.$and.push({ "object.id": activity._id });

            let baseQuery = {
                filter: actIdFilter,
                process: [
                    {
                        $frequentValues: {
                            path: "context.extensions['https://dcsa-user'].status",
                            limit: 10000,
                            metrics: {
                                count: {
                                    $range: "context.extensions['https://dcsa-user'].userid"
                                }
                            }
                        }
                    }
                ]
            }

            const innerStage = new QueryStage({
                filter: baseQuery.filter,
                process: baseQuery.process,
                lrs: lrs,
            });
            let innerResults = await Executor.runStage(innerStage, null, lrs);

            if (courseStatus === 'notCompleted') {
                // baseFilter with just comp
                const compQuery = clone(baseQuery);
                compQuery.filter.$and.push({ "verb.id": "http://adlnet.gov/expapi/verbs/completed" });
                const compStage = new QueryStage({
                    filter: compQuery.filter,
                    process: compQuery.process,
                    lrs: lrs,
                });
                const compResults = await Executor.runStage(compStage, null, lrs);

                // Value is just reg - just comp.
                innerResults = innerResults.map(reg => {
                    const comp = compResults.find(comp => comp._id === reg._id);
                    return { _id: reg._id, count: comp ? reg.count - comp.count : reg.count }
                });
            }

            for (const innerResult of innerResults) {
                row[innerResult._id] += innerResult.count;
                rowTotal += innerResult.count;

                grandTotal[innerResult._id] += innerResult.count;
                grandTotal['TOTAL'] += innerResult.count;
            }
        }

        // Add row totals.
        row['TOTAL'] = rowTotal;

        results.push(row);
    }

    if (canceled) return [];
    if (results.length <= 0) return [];

    // Add column totals.
    results[results.length] = grandTotal;

    console.log('time: ', Date.now() - timeStart);
    return results;
};
