/** ***********************************************************************
*
* Veracity Technology Consultants CONFIDENTIAL
* __________________
*
*  2019 Veracity Technology Consultants
*  All Rights Reserved.
*
* NOTICE:  All information contained herein is, and remains
* the property of Veracity Technology Consultants and its suppliers,
* if any.  The intellectual and technical concepts contained
* herein are proprietary to Veracity Technology Consultants
* and its suppliers and may be covered by U.S. and Foreign Patents,
* patents in process, and are protected by trade secret or copyright law.
* Dissemination of this information or reproduction of this material
* is strictly forbidden unless prior written permission is obtained
* from Veracity Technology Consultants.
*/
const QueryStage = require('../../analytics/engine/stages/QueryStage');
const Executor = require('../../analytics/engine/Executor');

module.exports.getUniqueValues = async (lrs, path, filter) => {
    const query = {
        filter: filter,
        process: [
            {
                $frequentValues: {
                    path: path,
                    limit: 10000,
                }
            },
            {
                $sort: { _id: 1 }
            }
        ]
    };

    const queryStage = new QueryStage({
        filter: query.filter,
        process: query.process,
        lrs: lrs,
    });

    const queryResults = await Executor.runStage(queryStage, null, lrs);

    return queryResults.map(r => r._id);
};

module.exports.getUniqueActivityDocs = async (lrs, filter) => {
    const activityQuery = {
        filter: filter,
        process: [
            {
                $frequentValues: {
                    path: "object.id",
                    limit: 10000,
                },
            },
            {
                $toCanonicalDisplay: { }
            },
            {
                $sort: { _id: 1 }
            }
        ]
    }

    const activityStage = new QueryStage({
        filter: activityQuery.filter,
        process: activityQuery.process,
        lrs: lrs,
    });
    const activities = await Executor.runStage(activityStage, null, lrs);

    return activities;
};

module.exports.getCompletedFilter = (courseStatus) => {
    switch (courseStatus) {
        case 'completed':
            return {
                $match: {
                    isComplete: "Completed"
                }
            };
        case 'notCompleted':
            return {
                $match: {
                    isComplete: "Not Completed"
                }
            };
        default:
            return null;
    }
}

module.exports.downloadTimeUpdate = async (iterationCount, total, start, file) => {
    file = await file.constructor.findOne({uuid:file.uuid});
    if(file.data == "cancel")
    {
        file.data == "canceled";
        await file.save();
        throw(new Error("canceled"));
    }else 
    {
        let now = Date.now();
        let elapsed = now - start;
        let totalEstimate = (1/(iterationCount/total)) * elapsed;
        let remaining = totalEstimate - elapsed;
        file.data = `(${(iterationCount/total).toFixed(2) * 100}% - about ${(remaining/60000).toFixed(2)} minutes remaining)`;    
        await file.save();
    }
}
